@@ -1,7 +1,7 @@
 use crate::devices::smartsocket::SmartSocket;
 use crate::devices::thermometer::Thermometer;
 
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 pub enum DeviceType {
     SmartSocket(SmartSocket),
     Thermometer(Thermometer),
